@@ -7,10 +7,12 @@ from test_project.test_app import documents
 class StrangePersonResource(resources.MongoEngineResource):
     class Meta:
         queryset = documents.StrangePerson.objects.all()
+        excludes = ('hidden',)
 
 class OtherStrangePersonResource(resources.MongoEngineResource):
     class Meta:
         queryset = documents.StrangePerson.objects.all()
+        excludes = ('hidden',)
 
 class PersonResource(resources.MongoEngineResource):
     class Meta:
@@ -19,6 +21,7 @@ class PersonResource(resources.MongoEngineResource):
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
         authorization = tastypie_authorization.Authorization()
         ordering = ('name',)
+        excludes = ('hidden',)
         paginator_class = paginator.Paginator
 
         polymorphic = {
@@ -26,11 +29,19 @@ class PersonResource(resources.MongoEngineResource):
             'strangeperson': StrangePersonResource,
         }
 
+class PersonObjectClassResource(resources.MongoEngineResource):
+    class Meta:
+        object_class = documents.Person
+        allowed_methods = ('get', 'post')
+        authorization = tastypie_authorization.Authorization()
+        resource_name = 'personobjectclass'
+
 class OnlySubtypePersonResource(resources.MongoEngineResource):
     class Meta:
         queryset = documents.Person.objects.all()
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
         authorization = tastypie_authorization.Authorization()
+        excludes = ('hidden',)
 
         polymorphic = {
             'strangeperson': StrangePersonResource,
@@ -46,6 +57,7 @@ class EmbeddedPersonResource(resources.MongoEngineResource):
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
         authorization = tastypie_authorization.Authorization()
         ordering = ('name',)
+        excludes = ('hidden',)
 
         polymorphic = {
             'person': 'self',
@@ -79,6 +91,21 @@ class BoardResource(resources.MongoEngineResource):
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
         authorization = tastypie_authorization.Authorization()
 
+class EmbeddedCommentWithIDResource(resources.MongoEngineResource):
+    class Meta:
+        object_class = documents.EmbeddedCommentWithID
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
+        paginator_class = paginator.Paginator
+
+class DocumentWithIDResource(resources.MongoEngineResource):
+    comments = fields.EmbeddedListField(of='test_project.test_app.api.resources.EmbeddedCommentWithIDResource', attribute='comments', full=True, null=True)
+
+    class Meta:
+        queryset = documents.DocumentWithID.objects.all()
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
+
 class EmbeddedListInEmbeddedDocTestResource(resources.MongoEngineResource):
     post = fields.EmbeddedDocumentField(embedded='test_project.test_app.api.resources.EmbeddedPostResource', attribute='post')
 
@@ -102,6 +129,8 @@ class DictFieldTestResource(resources.MongoEngineResource):
         authorization = tastypie_authorization.Authorization()
 
 class ListFieldTestResource(resources.MongoEngineResource):
+    extra_list = tastypie_fields.ListField()
+
     class Meta:
         queryset = documents.ListFieldTest.objects.all()
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
@@ -116,6 +145,30 @@ class EmbeddedListFieldTestResource(resources.MongoEngineResource):
         authorization = tastypie_authorization.Authorization()
         ordering = ('id', 'embeddedlist')
 
+class EmbeddedListFieldNonFullTestResource(resources.MongoEngineResource):
+    embeddedlist = fields.EmbeddedListField(of='test_project.test_app.api.resources.EmbeddedPersonResource', attribute='embeddedlist', full=False, null=True)
+
+    class Meta:
+        queryset = documents.EmbeddedListFieldTest.objects.all()
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
+
+class ReferencedListFieldTestResource(resources.MongoEngineResource):
+    referencedlist = fields.ReferencedListField(of='test_project.test_app.api.resources.PersonResource', attribute='referencedlist', full=True, null=True)
+
+    class Meta:
+        queryset = documents.ReferencedListFieldTest.objects.all()
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
+
+class ReferencedListFieldNonFullTestResource(resources.MongoEngineResource):
+    referencedlist = fields.ReferencedListField(of='test_project.test_app.api.resources.PersonResource', attribute='referencedlist', full=False, null=True)
+
+    class Meta:
+        queryset = documents.ReferencedListFieldTest.objects.all()
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
+
 class BooleanMapTestResource(resources.MongoEngineResource):
     is_published_defined = tastypie_fields.BooleanField(default=False, null=False, attribute='is_published_defined')
 
@@ -131,3 +184,11 @@ class EmbeddedListWithFlagFieldTestResource(resources.MongoEngineResource):
         queryset = documents.EmbeddedListWithFlagFieldTest.objects.all()
         allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
         authorization = tastypie_authorization.Authorization()
+
+class AutoAllocationFieldTestResource(resources.MongoEngineResource):
+    slug = tastypie_fields.CharField(readonly=True, attribute='slug')
+
+    class Meta:
+        queryset = documents.AutoAllocationFieldTest.objects.all()
+        allowed_methods = ('get', 'post', 'put', 'patch', 'delete')
+        authorization = tastypie_authorization.Authorization()
