@@ -1,3 +1,5 @@
+import bson
+
 import mongoengine
 
 class Person(mongoengine.Document):
@@ -7,6 +9,7 @@ class Person(mongoengine.Document):
 
     name = mongoengine.StringField(max_length=200, required=True)
     optional = mongoengine.StringField(max_length=200, required=False)
+    hidden = mongoengine.StringField(max_length=200, required=False)
 
 class StrangePerson(Person):
     strange = mongoengine.StringField(max_length=100, required=True)
@@ -14,6 +17,7 @@ class StrangePerson(Person):
 class EmbeddedPerson(mongoengine.EmbeddedDocument):
     name = mongoengine.StringField(max_length=200, required=True)
     optional = mongoengine.StringField(max_length=200, required=False)
+    hidden = mongoengine.StringField(max_length=200, required=False)
 
 class EmbeddedStrangePerson(EmbeddedPerson):
     strange = mongoengine.StringField(max_length=100, required=True)
@@ -32,6 +36,14 @@ class EmbeddedPost(mongoengine.EmbeddedDocument):
 class Board(mongoengine.Document):
     posts = mongoengine.ListField(mongoengine.EmbeddedDocumentField(EmbeddedPost))
 
+class EmbeddedCommentWithID(mongoengine.EmbeddedDocument):
+    id = mongoengine.ObjectIdField(primary_key=True, default=lambda: bson.ObjectId())
+    content = mongoengine.StringField(max_length=200, required=True)
+
+class DocumentWithID(mongoengine.Document):
+    title = mongoengine.StringField(max_length=200, required=True)
+    comments = mongoengine.ListField(mongoengine.EmbeddedDocumentField(EmbeddedCommentWithID))
+
 class EmbeddedListInEmbeddedDocTest(mongoengine.Document):
     post = mongoengine.EmbeddedDocumentField(EmbeddedPost)
 
@@ -49,6 +61,9 @@ class ListFieldTest(mongoengine.Document):
 class EmbeddedListFieldTest(mongoengine.Document):
     embeddedlist = mongoengine.ListField(mongoengine.EmbeddedDocumentField(EmbeddedPerson))
 
+class ReferencedListFieldTest(mongoengine.Document):
+    referencedlist = mongoengine.ListField(mongoengine.ReferenceField(Person))
+
 class BooleanMapTest(mongoengine.Document):
     is_published_auto = mongoengine.BooleanField(default=False, required=True)
     is_published_defined = mongoengine.BooleanField(default=False, required=True)
@@ -56,3 +71,13 @@ class BooleanMapTest(mongoengine.Document):
 class EmbeddedListWithFlagFieldTest(mongoengine.Document):
     embeddedlist = mongoengine.ListField(mongoengine.EmbeddedDocumentField(EmbeddedPerson))
     is_published = mongoengine.BooleanField(default=False, required=True)
+
+class AutoAllocationFieldTest(mongoengine.Document):
+    name = mongoengine.StringField(required=True)
+    slug = mongoengine.StringField(required=True)
+
+    def save(self, *args, **kwargs):
+        from django.template.defaultfilters import slugify
+        if not self.slug:
+            self.slug = slugify(self.name)
+        super(AutoAllocationFieldTest, self).save(*args, **kwargs)
